@@ -2,29 +2,40 @@ import datetime
 import typing
 from typing import List
 
+from lime_etl.adapters import (
+    timestamp_adapter,
+)
 from lime_etl.domain import job_spec, job_test_result, value_objects
-from lime_etl.services import job_logging_service, unit_of_work
+from lime_etl.services import admin_unit_of_work, job_logging_service
 
 
 class DeleteOldLogs(job_spec.AdminJobSpec):
     def __init__(
         self,
         days_to_keep: value_objects.DaysToKeep,
+        ts_adapter: timestamp_adapter.TimestampAdapter = timestamp_adapter.LocalTimestampAdapter(),
     ):
         self._days_to_keep = days_to_keep
+        self._ts_adapter = ts_adapter
 
     @property
     def dependencies(self) -> List[value_objects.JobName]:
         return []
 
-    @property
-    def flex_pct(self) -> value_objects.FlexPercent:
-        return value_objects.FlexPercent(0)
-
     @property
     def max_retries(self) -> value_objects.MaxRetries:
         return value_objects.MaxRetries(1)
 
+    def on_execution_error(
+        self, error_message: str
+    ) -> typing.Optional[job_spec.AdminJobSpec]:
+        return None
+
+    def on_test_failure(
+        self, test_results: typing.FrozenSet[job_test_result.JobTestResult]
+    ) -> typing.Optional[job_spec.AdminJobSpec]:
+        return None
+
     @property
     def job_name(self) -> value_objects.JobName:
         return value_objects.JobName("delete_old_logs")
@@ -39,55 +50,45 @@ class DeleteOldLogs(job_spec.AdminJobSpec):
 
     def run(
         self,
-        uow: unit_of_work.UnitOfWork,
-        logger: job_logging_service.JobLoggingService,
+        admin_uow: admin_unit_of_work.AdminUnitOfWork,
+        logger: job_logging_service.AbstractJobLoggingService,
     ) -> value_objects.Result:
-        with uow:
-            uow.batch_log.delete_old_entries(days_to_keep=self._days_to_keep)
-            uow.commit()
-
-        logger.log_info(
-            message=value_objects.LogMessage(
+        with admin_uow as uow:
+            uow.batch_log_repo.delete_old_entries(days_to_keep=self._days_to_keep)
+            logger.log_info(
                 f"Deleted batch log entries older than {self._days_to_keep.value} days old."
             )
-        )
 
-        with uow:
-            uow.job_log.delete_old_entries(days_to_keep=self._days_to_keep)
-            uow.commit()
-
-        logger.log_info(
-            message=value_objects.LogMessage(
+            uow.job_log_repo.delete_old_entries(days_to_keep=self._days_to_keep)
+            uow.save()
+            logger.log_info(
                 f"Deleted job log entries older than {self._days_to_keep.value} days old."
             )
-        )
 
-        with uow:
-            uow.batches.delete_old_entries(days_to_keep=self._days_to_keep)
-            uow.commit()
-
-        logger.log_info(
-            message=value_objects.LogMessage(
+            uow.batch_repo.delete_old_entries(self._days_to_keep)
+            admin_uow.save()
+            logger.log_info(
                 f"Deleted batch results older than {self._days_to_keep.value} days old."
             )
-        )
 
         return value_objects.Result.success()
 
     def test(
         self,
-        uow: unit_of_work.UnitOfWork,
-        logger: job_logging_service.JobLoggingService,
+        admin_uow: admin_unit_of_work.AdminUnitOfWork,
+        logger: job_logging_service.AbstractJobLoggingService,
     ) -> typing.Collection[job_test_result.SimpleJobTestResult]:
-        with uow:
-            ts = uow.ts_adapter.now().value
-            cutoff_date = datetime.datetime.combine(
-                (ts - datetime.timedelta(days=self._days_to_keep.value)).date(),
-                datetime.datetime.min.time(),
-            )
-            earliest_log_entry = uow.batch_log.get_earliest()
+        cutoff_date = datetime.datetime.combine(
+            (
+                self._ts_adapter.now().value
+                - datetime.timedelta(days=self._days_to_keep.value)
+            ).date(),
+            datetime.datetime.min.time(),
+        )
+        with admin_uow as uow:
+            earliest_ts = uow.batch_log_repo.get_earliest_timestamp()
 
-        if earliest_log_entry.ts.value < cutoff_date:
+        if earliest_ts and earliest_ts < cutoff_date:
             return [
                 job_test_result.SimpleJobTestResult(
                     test_name=value_objects.TestName(
@@ -95,7 +96,7 @@ class DeleteOldLogs(job_spec.AdminJobSpec):
                     ),
                     test_success_or_failure=value_objects.Result.failure(
                         f"The earliest batch log entry is from "
-                        f"{earliest_log_entry.ts.value.strftime('%Y-%m-%d %H:%M:%S')}"
+                        f"{earliest_ts.strftime('%Y-%m-%d %H:%M:%S')}"
                     ),
                 )
             ]
